package com.ningdatech.pmapi.projectdeclared.utils;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.pmapi.common.constant.BizConst;
import com.ningdatech.pmapi.common.constant.RegionConst;
import com.ningdatech.pmapi.gov.contants.BizProjectContant;
import com.ningdatech.pmapi.gov.model.dto.*;
import com.ningdatech.pmapi.gov.model.entity.GovBizProjectBaseinfo;
import com.ningdatech.pmapi.gov.model.entity.GovOperationProjectBaseinfo;
import com.ningdatech.pmapi.gov.service.IGovBizProjectBaseinfoService;
import com.ningdatech.pmapi.gov.service.IGovOperationProjectBaseinfoService;
import com.ningdatech.pmapi.projectdeclared.contants.ProjectCodeContant;
import com.ningdatech.pmapi.projectlib.model.dto.ProjectDTO;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectCollection;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import java.time.LocalDateTime;
import java.util.Objects;

/**
 * 生成项目编号工具类
 *
 * @author CMM
 * @since 2023/04/04 14:13
 */
@Component
public class GenerateProjectCodeUtil {

    @Autowired
    private IProjectService projectService;

    @Autowired
    private IGovBizProjectBaseinfoService baseinfoService;

    @Autowired
    private IGovOperationProjectBaseinfoService operationProjectBaseinfoService;

    public String generateProjectCode(ProjectDTO project){
        // 获取所属行政区划代码(9位)
        String areaCode = project.getAreaCode() + ProjectCodeContant.REGION_LAST_3;
        // 获取建设年度 (10-13)
        String year = Objects.nonNull(project.getProjectYear()) ? String.valueOf(project.getProjectYear())
                : String.valueOf(LocalDateTime.now().getYear());
        // 14-15 项目类型
        String projectType =  project.getProjectType();
        // 16-20 项目序号00001
        String currentCode = areaCode + year + projectType;
        // 16-20 项目序号00001
        //要查询 两边的项目序号
        Long max = 0L;
        GovBizProjectBaseinfo baseMax = baseinfoService.getOne(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .like(GovBizProjectBaseinfo::getBaseProjId, currentCode)
                .orderByDesc(GovBizProjectBaseinfo::getBaseProjId)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(baseMax) && StringUtils.isNotBlank(baseMax.getBaseProjId())){
            max = Long.valueOf(baseMax.getBaseProjId().substring(baseMax.getBaseProjId().length() - 5));
        }
        GovOperationProjectBaseinfo operationBaseMax = operationProjectBaseinfoService.getOne(Wrappers.lambdaQuery(GovOperationProjectBaseinfo.class)
                .like(GovOperationProjectBaseinfo::getBaseProjId, currentCode)
                .orderByDesc(GovOperationProjectBaseinfo::getBaseProjId)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(operationBaseMax) && StringUtils.isNotBlank(operationBaseMax.getBaseProjId())){
            Long operationMax = Long.valueOf(baseMax.getBaseProjId().substring(baseMax.getBaseProjId().length() - 5));
            max = Math.max(max,operationMax);
        }
        Project projectMax = projectService.getOne(Wrappers.lambdaQuery(Project.class)
                .like(Project::getProjectCode, currentCode)
                .orderByDesc(Project::getProjectCode)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(projectMax) && StringUtils.isNotBlank(projectMax.getProjectCode())){
            Long projectMaxCode = Long.valueOf(projectMax.getProjectCode().substring(projectMax.getProjectCode().length() - 5));
            max = Math.max(max,projectMaxCode);
        }
        max = max + 1;
        // 生成20位的项目编号
        return currentCode + String.format(ProjectCodeContant.SHUZI_5, max);
    }

    public String generateProjectCode(ProjectCollection project){
        // 获取所属行政区划代码(9位)
        String areaCode = project.getAreaCode() + ProjectCodeContant.REGION_LAST_3;
        // 获取建设年度 (10-13)
        String year = Objects.nonNull(project.getProjectYear()) ? String.valueOf(project.getProjectYear())
                : String.valueOf(LocalDateTime.now().getYear());
        // 14-15 项目类型
        String projectType =  ProjectCodeContant.PROJECT_TYPE_PREFIX + project.getProjectType();
        // 16-20 项目序号00001
        String currentCode = areaCode + year + projectType;
        // 16-20 项目序号00001
        //要查询 两边的项目序号
        Long max = 0L;
        GovBizProjectBaseinfo baseMax = baseinfoService.getOne(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .like(GovBizProjectBaseinfo::getBaseProjId, currentCode)
                .orderByDesc(GovBizProjectBaseinfo::getBaseProjId)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(baseMax) && StringUtils.isNotBlank(baseMax.getBaseProjId())){
            max = Long.valueOf(baseMax.getBaseProjId().substring(baseMax.getBaseProjId().length() - 5));
        }
        GovOperationProjectBaseinfo operationBaseMax = operationProjectBaseinfoService.getOne(Wrappers.lambdaQuery(GovOperationProjectBaseinfo.class)
                .like(GovOperationProjectBaseinfo::getBaseProjId, currentCode)
                .orderByDesc(GovOperationProjectBaseinfo::getBaseProjId)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(operationBaseMax) && StringUtils.isNotBlank(operationBaseMax.getBaseProjId())){
            Long operationMax = Long.valueOf(baseMax.getBaseProjId().substring(baseMax.getBaseProjId().length() - 5));
            max = Math.max(max,operationMax);
        }
        Project projectMax = projectService.getOne(Wrappers.lambdaQuery(Project.class)
                .like(Project::getProjectCode, currentCode)
                .orderByDesc(Project::getProjectCode)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(projectMax) && StringUtils.isNotBlank(projectMax.getProjectCode())){
            Long projectMaxCode = Long.valueOf(projectMax.getProjectCode().substring(projectMax.getProjectCode().length() - 5));
            max = Math.max(max,projectMaxCode);
        }
        max = max + 1;
        // 生成20位的项目编号
        return currentCode + String.format(ProjectCodeContant.SHUZI_5, max);
    }

    public String generateProjectCode(GovBizProjectSaveDTO dto) {
        // 获取所属行政区划代码(9位)
        GovBizProjectBaseinfoDTO baseinfo = dto.getBaseinfo();
        GovBizProjectApplyDTO apply = dto.getApply();

        VUtils.isTrue(Objects.isNull(baseinfo)).throwMessage("没有项目基本信息 无法生成项目编号!");
        VUtils.isTrue(Objects.isNull(apply)).throwMessage("没有项目申报信息 无法生成项目编号!");

        String areaCode = (StringUtils.isNotBlank(baseinfo.getBaseAreaCode()) ? baseinfo.getBaseAreaCode() : RegionConst.RC_LS + BizConst.NINE_AREA_CODE_LAST);
        // 获取建设年度 (10-13)
        String year = Objects.nonNull(apply.getBaseProjSetYear()) ? apply.getBaseProjSetYear()
                : String.valueOf(LocalDateTime.now().getYear());
        // 14-15 项目类型
        String projectType = StringUtils.isNotBlank(baseinfo.getBaseProjType()) ? baseinfo.getBaseProjType() :
                BizProjectContant.ProjectCollection.IS_EFFECTIVE;
        String currentCode = areaCode + year + projectType;
        // 16-20 项目序号00001
        //要查询 两边的项目序号
        Long max = 0L;
        GovBizProjectBaseinfo baseMax = baseinfoService.getOne(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .like(GovBizProjectBaseinfo::getBaseProjId, currentCode)
                .orderByDesc(GovBizProjectBaseinfo::getBaseProjId)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(baseMax) && StringUtils.isNotBlank(baseMax.getBaseProjId())){
            max = Long.valueOf(baseMax.getBaseProjId().substring(baseMax.getBaseProjId().length() - 5));
        }
        GovOperationProjectBaseinfo operationBaseMax = operationProjectBaseinfoService.getOne(Wrappers.lambdaQuery(GovOperationProjectBaseinfo.class)
                .like(GovOperationProjectBaseinfo::getBaseProjId, currentCode)
                .orderByDesc(GovOperationProjectBaseinfo::getBaseProjId)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(operationBaseMax) && StringUtils.isNotBlank(operationBaseMax.getBaseProjId())){
            Long operationMax = Long.valueOf(baseMax.getBaseProjId().substring(baseMax.getBaseProjId().length() - 5));
            max = Math.max(max,operationMax);
        }
        Project projectMax = projectService.getOne(Wrappers.lambdaQuery(Project.class)
                .like(Project::getProjectCode, currentCode)
                .orderByDesc(Project::getProjectCode)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(projectMax) && StringUtils.isNotBlank(projectMax.getProjectCode())){
            Long projectMaxCode = Long.valueOf(projectMax.getProjectCode().substring(projectMax.getProjectCode().length() - 5));
            max = Math.max(max,projectMaxCode);
        }
        max = max + 1;
        // 生成20位的项目编号
        return currentCode + String.format(ProjectCodeContant.SHUZI_5, max);
    }

    public String generateProjectCode(GovOperationProjectSaveDTO dto) {
        // 获取所属行政区划代码(9位)
        GovOperationProjectBaseinfoDTO baseinfo = dto.getBaseinfo();
        GovBizProjectApplyDTO apply = dto.getApply();

        VUtils.isTrue(Objects.isNull(baseinfo)).throwMessage("没有项目基本信息 无法生成项目编号!");
        VUtils.isTrue(Objects.isNull(apply)).throwMessage("没有项目申报信息 无法生成项目编号!");

        String areaCode = (StringUtils.isNotBlank(baseinfo.getBaseAreaCode()) ? baseinfo.getBaseAreaCode() : RegionConst.RC_LS + BizConst.NINE_AREA_CODE_LAST);
        // 获取建设年度 (10-13)
        String year = Objects.nonNull(apply.getBaseProjSetYear()) ? apply.getBaseProjSetYear()
                : String.valueOf(LocalDateTime.now().getYear());
        // 14-15 项目类型
        String projectType = StringUtils.isNotBlank(baseinfo.getBaseProjType()) ? baseinfo.getBaseProjType() :
                BizProjectContant.ProjectCollection.IS_EFFECTIVE;
        String currentCode = areaCode + year + projectType;
        // 16-20 项目序号00001
        //要查询 两边的项目序号
        Long max = 0L;
        GovBizProjectBaseinfo baseMax = baseinfoService.getOne(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .like(GovBizProjectBaseinfo::getBaseProjId, currentCode)
                .orderByDesc(GovBizProjectBaseinfo::getBaseProjId)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(baseMax) && StringUtils.isNotBlank(baseMax.getBaseProjId())){
            max = Long.valueOf(baseMax.getBaseProjId().substring(baseMax.getBaseProjId().length() - 5));
        }
        GovOperationProjectBaseinfo operationBaseMax = operationProjectBaseinfoService.getOne(Wrappers.lambdaQuery(GovOperationProjectBaseinfo.class)
                .like(GovOperationProjectBaseinfo::getBaseProjId, currentCode)
                .orderByDesc(GovOperationProjectBaseinfo::getBaseProjId)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(operationBaseMax) && StringUtils.isNotBlank(operationBaseMax.getBaseProjId())){
            Long operationMax = Long.valueOf(baseMax.getBaseProjId().substring(baseMax.getBaseProjId().length() - 5));
            max = Math.max(max,operationMax);
        }
        Project projectMax = projectService.getOne(Wrappers.lambdaQuery(Project.class)
                .like(Project::getProjectCode, currentCode)
                .orderByDesc(Project::getProjectCode)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(projectMax) && StringUtils.isNotBlank(projectMax.getProjectCode())){
            Long projectMaxCode = Long.valueOf(projectMax.getProjectCode().substring(projectMax.getProjectCode().length() - 5));
            max = Math.max(max,projectMaxCode);
        }
        max = max + 1;
        // 生成20位的项目编号
        return currentCode + String.format(ProjectCodeContant.SHUZI_5, max);
    }
}
