package com.ningdatech.pmapi.scheduler.task;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.StopWatch;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.ningdatech.pmapi.common.constant.BizConst;
import com.ningdatech.pmapi.common.constant.RegionConst;
import com.ningdatech.pmapi.common.helper.RegionCacheHelper;
import com.ningdatech.pmapi.common.statemachine.contants.RegionContant;
import com.ningdatech.pmapi.common.util.BizUtils;
import com.ningdatech.pmapi.dashboard.constant.DashboardConstant;
import com.ningdatech.pmapi.dashboard.model.entity.CockpitStats;
import com.ningdatech.pmapi.dashboard.model.po.SecrecyPasswordGradePO;
import com.ningdatech.pmapi.dashboard.service.ICockpitStatsService;
import com.ningdatech.pmapi.expert.entity.ExpertUserFullInfo;
import com.ningdatech.pmapi.expert.service.IExpertUserFullInfoService;
import com.ningdatech.pmapi.gov.enumeration.GovProjectStatusEnum;
import com.ningdatech.pmapi.gov.model.entity.GovBizProjectApply;
import com.ningdatech.pmapi.gov.model.entity.GovBizProjectApprove;
import com.ningdatech.pmapi.gov.model.entity.GovBizProjectBaseinfo;
import com.ningdatech.pmapi.gov.model.entity.GovOperationProjectBaseinfo;
import com.ningdatech.pmapi.gov.service.IGovBizProjectApplyService;
import com.ningdatech.pmapi.gov.service.IGovBizProjectApproveService;
import com.ningdatech.pmapi.gov.service.IGovBizProjectBaseinfoService;
import com.ningdatech.pmapi.gov.service.IGovOperationProjectBaseinfoService;
import com.ningdatech.pmapi.performance.model.entity.PerformanceAppraisalProject;
import com.ningdatech.pmapi.performance.service.IPerformanceAppraisalProjectService;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.sys.model.dto.RegionDTO;
import com.ningdatech.pmapi.sys.model.entity.WflowEarlyWarningRecords;
import com.ningdatech.pmapi.sys.service.IEarlyWarningRecordsService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import java.math.BigDecimal;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author ZPF
 * @Class CockpitStatsStatisticsTask
 * 驾驶舱 统计 每日任务
 * @since 2023/08/31 18:16
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class CockpitStatsStatisticsTask {

    @Value("${hostname}")
    public String HOST;

    @Value("${spring.profiles.active}")
    public String active;

    @Autowired
    private ICockpitStatsService cockpitStatsService;

    @Autowired
    private RegionCacheHelper regionCacheHelper;

    @Autowired
    private IProjectService projectService;

    @Autowired
    private IExpertUserFullInfoService expertUserFullInfoService;

    @Autowired
    private IPerformanceAppraisalProjectService performanceAppraisalProjectService;
    @Autowired
    private IGovBizProjectBaseinfoService baseinfoService;

    @Autowired
    private IGovBizProjectApplyService applyService;

    @Autowired
    private IGovBizProjectApproveService approveService;

    @Autowired
    private IGovOperationProjectBaseinfoService operationBaseInfoService;

    @Autowired
    private IEarlyWarningRecordsService earlyWarningRecordsService;

    private List<Integer> years = Lists.newArrayList(2021,2022,2023,2024,2025);

    /**
     * 定义统计 驾驶舱数据 每天3点开始执行一次
     *
     */
    @Scheduled(cron = "0 0 3 * * ?")
    public void doTask() throws UnknownHostException {
        if (!HOST.equals(InetAddress.getLocalHost().getHostName())) {
            log.info("定时器没开启或者host不对! {}:{}",
                    HOST,InetAddress.getLocalHost().getHostName());
            return;
        }
        if(BizConst.PRE.equals(active)){
            log.info("预发环境不用同步!");
            return;
        }

        log.info("驾驶舱数据统计任务开始=====");

        StopWatch stopWatch = new StopWatch();
        stopWatch.start();

        List<CockpitStats> res = Lists.newArrayList();
        //1.根据2个维度来统计 区域和分年
        //1.总的
        res.add(statisticsData(DashboardConstant.CockpitStats.TOTAL,
                DashboardConstant.CockpitStats.TOTAL,DashboardConstant.CockpitStats.NONE_YEAR));
        for(Integer year : years){
            res.add(statisticsData(DashboardConstant.CockpitStats.TOTAL,
                    DashboardConstant.CockpitStats.TOTAL,year));
        }

        List<RegionDTO> regions = regionCacheHelper.listChildren(RegionConst.RC_LS, RegionConst.RL_CITY);
        regions = regions.stream().filter(r -> r.getRegionLevel() != RegionConst.RL_CITY).collect(Collectors.toList());
        for(RegionDTO regionDto :  regions){
            res.add(statisticsData(regionDto.getRegionCode(),regionDto.getRegionName(),
                    DashboardConstant.CockpitStats.NONE_YEAR));
            for(Integer year : years){
                res.add(statisticsData(regionDto.getRegionCode(),regionDto.getRegionName(),year));
            }
        }

        //如果结果不为空 就删除之前的 插入最新的数据
        if(CollUtil.isNotEmpty(res)){
            //计算增长率
            computeRates(res);
            List<CockpitStats> list = cockpitStatsService.list();
            List<Long> ids = list.stream().map(CockpitStats::getId).collect(Collectors.toList());
            cockpitStatsService.removeByIds(ids);
            cockpitStatsService.saveBatch(res);
        }

        stopWatch.stop();
        log.info("驾驶舱数据统计任务结束====={}s",stopWatch.getTotalTimeSeconds());
    }

    private void computeRates(List<CockpitStats> res) {
        Map<String, CockpitStats> map = res.stream().collect(Collectors.toMap(r -> r.getRegionCode() + r.getYear(), r -> r));
        for(CockpitStats cockpitStats : res){
            if(Objects.isNull(cockpitStats.getYear())){
                continue;
            }
            //算增长率
            //去年的数据 用于算 增长比例
            if(cockpitStats.getYear().equals(DashboardConstant.CockpitStats.NONE_YEAR)){
                Integer thisYear = LocalDateTime.now().getYear();
                CockpitStats thisYearData = map.get(cockpitStats.getRegionCode() + thisYear);
                CockpitStats lastYearData = map.get(cockpitStats.getRegionCode() + (thisYear - 1));
                buidMonitorRise(cockpitStats,thisYearData,lastYearData);
            }else{
                CockpitStats lastYearData = map.get(cockpitStats.getRegionCode() + (cockpitStats.getYear() - 1));
                buidMonitorRise(cockpitStats,cockpitStats,lastYearData);
            }
        }

    }

    /**
     * 统计的具体逻辑
     * @param regionCode
     * @param regionName
     * @param year
     * @return
     */
    private CockpitStats statisticsData(String regionCode, String regionName, Integer year) {
        CockpitStats cockpitStats = new CockpitStats();
        cockpitStats.setRegionCode(regionCode);
        cockpitStats.setRegionName(regionName);
        cockpitStats.setYear(year);

        //申报项目
        List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getNewest, Boolean.TRUE)
                .eq(StringUtils.isNotBlank(regionCode) && !DashboardConstant.CockpitStats.TOTAL.equals(regionCode),
                        Project::getAreaCode, regionCode)
                .eq(Objects.nonNull(year) && !year.equals(DashboardConstant.CockpitStats.NONE_YEAR),
                        Project::getProjectYear, year)
                .in(Objects.isNull(year) && year.equals(DashboardConstant.CockpitStats.NONE_YEAR),
                        Project::getProjectYear, years));

        //项目归集
        List<GovBizProjectBaseinfo> baseInfos = baseinfoService.list(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .eq(GovBizProjectBaseinfo::getDeleted, Boolean.FALSE)
                .eq(StringUtils.isNotBlank(regionCode) && !DashboardConstant.CockpitStats.TOTAL.equals(regionCode),
                        GovBizProjectBaseinfo::getBaseAreaCode, regionCode + RegionContant.MORE_CODE));

        List<GovBizProjectApprove> approves = Lists.newArrayList();
        List<GovBizProjectApply> applies = Lists.newArrayList();
        if(CollUtil.isNotEmpty(baseInfos)){
            List<String> baseProjIds = baseInfos.stream().map(GovBizProjectBaseinfo::getBaseProjId).collect(Collectors.toList());
            approves = approveService.list(Wrappers.lambdaQuery(GovBizProjectApprove.class)
                    .in(GovBizProjectApprove::getBaseProjId, baseProjIds));

            applies = applyService.list(Wrappers.lambdaQuery(GovBizProjectApply.class)
                    .in(GovBizProjectApply::getBaseProjId, baseProjIds));
        }


        //运维备案
        List<GovOperationProjectBaseinfo> operationBaseInfos = operationBaseInfoService.list(Wrappers.lambdaQuery(GovOperationProjectBaseinfo.class)
                .eq(GovOperationProjectBaseinfo::getDeleted, Boolean.FALSE)
                .eq(StringUtils.isNotBlank(regionCode) && !DashboardConstant.CockpitStats.TOTAL.equals(regionCode),
                        GovOperationProjectBaseinfo::getBaseAreaCode, regionCode + RegionContant.MORE_CODE));

        List<String> baseProjIds = baseInfos.stream().map(GovBizProjectBaseinfo::getBaseProjId)
                .collect(Collectors.toList());
        List<String> operationBaseProjIds = operationBaseInfos.stream().map(GovOperationProjectBaseinfo::getBaseProjId)
                .collect(Collectors.toList());
        Integer baseProjCount = 0;
        if(CollUtil.isNotEmpty(baseProjIds) || CollUtil.isNotEmpty(operationBaseProjIds)){
            baseProjIds.addAll(operationBaseProjIds);
            Long count = applyService.count(Wrappers.lambdaQuery(GovBizProjectApply.class)
                    .in(GovBizProjectApply::getBaseProjId, baseProjIds)
                    .eq(Objects.nonNull(year) && !year.equals(DashboardConstant.CockpitStats.NONE_YEAR),
                            GovBizProjectApply::getBaseProjSetYear, year)
                    .in(Objects.isNull(year) && year.equals(DashboardConstant.CockpitStats.NONE_YEAR),
                            GovBizProjectApply::getBaseProjSetYear, years));
            baseProjCount = count.intValue();
        }

        //1.项目监测
        //1.1 超期在建项目
        Integer overdueConstructionProjectsNum = projects.stream().filter(p -> {
            if(Objects.nonNull(p.getPlanAcceptanceTime()) &&
                    p.getPlanAcceptanceTime().compareTo(LocalDateTime.now()) < 0 &&
                    Objects.nonNull(p.getStatus()) && p.getStatus().compareTo(ProjectStatusEnum.ACCEPTED.getCode()) < 0){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList()).size();
        cockpitStats.setMonitorOverdueConstructionProjectsNum(overdueConstructionProjectsNum);
        //1.2 环节滞后项目
        cockpitStats.setMonitorLaggingProjectsNum(computeLagging(projects));

        //1.3预审驳回项目
        Integer preFailed = projects.stream().filter(p -> {
            if(Objects.nonNull(p.getStatus()) &&
                    p.getStatus().equals(ProjectStatusEnum.PREQUALIFICATION_FAILED.getCode())){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList()).size();
        cockpitStats.setMonitorRejectedPreReviewProjectsNum(preFailed);

        //1.4 建设方案评审失败
        Integer constructionFailed = projects.stream().filter(p -> {
            if(Objects.nonNull(p.getStatus()) && p.getStatus().equals(ProjectStatusEnum.SCHEME_REVIEW_FAILED.getCode())){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList()).size();
        cockpitStats.setMonitorRejectedApproveProjectsNum(constructionFailed);
        //1.5 验收不达标
        cockpitStats.setMonitorAcceptConditionsNotStandardsNum(0);

        //1.6 总申报金额
        Double totalDeclaredAmount = projects.stream().mapToDouble(p -> Objects.nonNull(p.getDeclareAmount()) ? p.getDeclareAmount().doubleValue() : 0.0).sum();
        cockpitStats.setMonitorDeclaredAmount(BigDecimal.valueOf(totalDeclaredAmount));
        //1.7 总安全投入
        Double totalSafytyAmount = projects.stream().mapToDouble(p -> {
            if(Objects.nonNull(p.getSafetyInputDescribe())){
                final Double[] total = {0.0};
                try{
                    JSONArray array = JSON.parseArray(p.getSafetyInputDescribe());
                    array.forEach(j -> {
                        JSONObject json = JSON.parseObject(JSON.toJSONString(j));
                        Double safetyInputAmount = json.getDouble(DashboardConstant.Protrait.FEILD_SAFETYMONEY);
                        total[0] += safetyInputAmount;
                    });
                }catch (Exception e){

                }
                return total[0];
            }
            return 0.0;
        }).sum();
        cockpitStats.setMonitorSafetyInputAmount(BigDecimal.valueOf(totalSafytyAmount));

        //1.8等保密评
        SecrecyPasswordGradePO secrecyPasswordGradePo = cockpitStatsService.getSecrecyPasswordGrade(regionCode, year);
        if(Objects.nonNull(secrecyPasswordGradePo)){
            cockpitStats.setMonitorSecrecyGrade1Num(secrecyPasswordGradePo.getMonitorSecrecyGrade1Num());
            cockpitStats.setMonitorSecrecyGrade2Num(secrecyPasswordGradePo.getMonitorSecrecyGrade2Num());
            cockpitStats.setMonitorSecrecyGrade3Num(secrecyPasswordGradePo.getMonitorSecrecyGrade3Num());
            cockpitStats.setMonitorSecrecyGrade4Num(secrecyPasswordGradePo.getMonitorSecrecyGrade4Num());
            cockpitStats.setMonitorSecrecyGrade5Num(secrecyPasswordGradePo.getMonitorSecrecyGrade5Num());
            cockpitStats.setMonitorPasswordGrade1Num(secrecyPasswordGradePo.getMonitorPasswordGrade1Num());
            cockpitStats.setMonitorPasswordGrade2Num(secrecyPasswordGradePo.getMonitorPasswordGrade2Num());
            cockpitStats.setMonitorPasswordGrade3Num(secrecyPasswordGradePo.getMonitorPasswordGrade3Num());
            cockpitStats.setMonitorPasswordGrade4Num(secrecyPasswordGradePo.getMonitorPasswordGrade4Num());
            cockpitStats.setMonitorPasswordGrade5Num(secrecyPasswordGradePo.getMonitorPasswordGrade5Num());
        }

        //2.专家统计
        List<ExpertUserFullInfo> experts = expertUserFullInfoService.list(Wrappers.lambdaQuery(ExpertUserFullInfo.class)
                .eq(StringUtils.isNotBlank(regionCode) && !DashboardConstant.CockpitStats.TOTAL.equals(regionCode),
                        ExpertUserFullInfo::getRegionCode, regionCode));
        cockpitStats.setExpertTotalNum(experts.size());
        cockpitStats.setExpertFinancialNum(0);
        cockpitStats.setExpertNetworkSecurityNum(0);
        cockpitStats.setExpertXinchuangNum(0);
        cockpitStats.setExpertPlanRationalityNum(0);
        cockpitStats.setExpertPromotionInfoTechnologyNum(0);
        cockpitStats.setExpertPartyGovInfoNum(0);
        cockpitStats.setExpertSoftHardPricingNum(0);
        cockpitStats.setExpertTechnicalFeasibilityAssessmentNum(0);

        //3.顶部数据
        //3.1 计划项目数(通过单位内部审核的项目)
        Integer planProjectNum = projects.stream().filter(p -> {
            if(Objects.nonNull(p.getStatus()) && p.getStatus().compareTo(ProjectStatusEnum.PENDING_PREQUALIFICATION.getCode()) >= 0){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList()).size();
        cockpitStats.setTopPlanProjectsNum(planProjectNum);
        //3.2 批复项目数(可以用批复金额来判断)
        Integer approvalNumDeclared = projects.stream().filter(p -> {
            if(Objects.nonNull(p.getApprovalAmount())){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList()).size();
        Integer approvalNumGov = baseInfos.stream().filter(p -> {
            if(Objects.nonNull(p.getBaseProjSetProg()) && Lists.newArrayList(GovProjectStatusEnum.APPROVED.getCode(),
                    GovProjectStatusEnum.PURCHASED.getCode(),GovProjectStatusEnum.HAS_PRE_INS.getCode(),
                    GovProjectStatusEnum.HAS_FINAL_INS.getCode()).contains(p.getBaseProjSetProg())){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList()).size();

        Double approvalAmount = projects.stream().filter(p -> {
            if(Objects.nonNull(p.getApprovalAmount())){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).mapToDouble(p -> p.getApprovalAmount().doubleValue()).sum();

        Double approvalAmountGov = approves.stream().filter(a -> {
            if(Objects.nonNull(a.getBaseInitialReviewTotalMoney())){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).mapToDouble(a -> a.getBaseInitialReviewTotalMoney().doubleValue()).sum();

        cockpitStats.setTopApprovalProjectsNum(approvalNumDeclared + approvalNumGov);
        cockpitStats.setTopApprovalProjectsAmount(BigDecimal.valueOf(approvalAmount).add(BigDecimal.valueOf(approvalAmountGov)));
        //3.3 平均建设周期
        OptionalDouble od = Stream.concat(projects.stream().map(Project::getBuildCycle),
                applies.stream().map(GovBizProjectApply::getBaseProjDuration)).mapToInt(c -> {
            try {
                Integer buildCycle = Integer.valueOf(c);
                return buildCycle;
            } catch (Exception e) {
                return 0;
            }
        }).average();
        Double average = od.isPresent() ? od.getAsDouble() : 0.0;
        cockpitStats.setTopAverageConstructionPeriod(average.intValue());
        //3.4 在建项目数 采购备案后的项目
        Integer contructionNum = projects.stream().filter(p -> {
            if(Objects.nonNull(p.getStatus()) &&
                    p.getStatus().compareTo(ProjectStatusEnum.UNDER_CONSTRUCTION.getCode()) >= 0
                && p.getStatus().compareTo(ProjectStatusEnum.ACCEPTED.getCode()) < 0){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList()).size();
        Integer contructionNumGov = baseInfos.stream().filter(b -> {
            if(Objects.nonNull(b.getBaseProjSetProg()) &&
                    Lists.newArrayList(GovProjectStatusEnum.PURCHASED.getCode(),GovProjectStatusEnum.HAS_PRE_INS.getCode()).contains(b.getBaseProjSetProg())){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList()).size();
        cockpitStats.setTopOngoingProjectsNum(contructionNum + contructionNumGov);

        //4.地图 只要放总数 取的时候 会有逻辑(也要取项目归集)
        cockpitStats.setProjectsTotal(projects.size() + baseProjCount);

        //5.下面项目状态数
        //5.1 处于计划 (单位内部)
        Integer inPlanNum = projects.stream().filter(p -> {
            if(Objects.nonNull(p.getStatus()) &&
                    p.getStatus().equals(ProjectStatusEnum.UNDER_INTERNAL_AUDIT.getCode())){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList()).size();
        cockpitStats.setProjectsTotalPlan(inPlanNum);

        //5.2 处于申报 (预审)
        Integer inApplyNum = projects.stream().filter(p -> {
            if(Objects.nonNull(p.getStatus()) &&
                    p.getStatus().compareTo(ProjectStatusEnum.UNDER_INTERNAL_AUDIT.getCode()) > 0
                    && p.getStatus().compareTo(ProjectStatusEnum.TO_BE_APPROVED.getCode()) <= 0){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList()).size();
        cockpitStats.setProjectsTotalApply(inApplyNum);

        //5.3 处于审批 (建设方案)
        Integer inApproveNum = projects.stream().filter(p -> {
            if(Objects.nonNull(p.getStatus()) && p.getStatus().equals(ProjectStatusEnum.SCHEME_UNDER_REVIEW.getCode())){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList()).size();
        cockpitStats.setProjectsTotalApprove(inApproveNum);
        //5.4 建设 (建设中)
        cockpitStats.setProjectsTotalConstruction(contructionNum + contructionNumGov);

        //5.5 验收 和 绩效 可以一起查
        List<Project> inAccpetProjects = projects.stream().filter(p -> {
            if(Objects.nonNull(p.getStatus()) &&
                    p.getStatus().equals(ProjectStatusEnum.ACCEPTED.getCode())){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList());
        List<GovBizProjectBaseinfo> inAccpetProjectsGov = baseInfos.stream().filter(b -> {
            if(Objects.nonNull(b.getBaseProjSetProg()) &&
                    b.getBaseProjSetProg().equals(GovProjectStatusEnum.HAS_FINAL_INS.getCode())){
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList());

        List<String> inAccpetCodes = inAccpetProjects.stream().map(Project::getProjectCode).collect(Collectors.toList());
        List<PerformanceAppraisalProject> performances = performanceAppraisalProjectService.list(Wrappers.lambdaQuery(PerformanceAppraisalProject.class)
                .in(CollUtil.isNotEmpty(inAccpetCodes),PerformanceAppraisalProject::getProjectCode, inAccpetCodes)
                .in(CollUtil.isEmpty(inAccpetCodes),PerformanceAppraisalProject::getProjectCode, Lists.newArrayList("NONE"))
                .isNotNull(PerformanceAppraisalProject::getVerifyTotalScore));
        cockpitStats.setProjectsTotalAccept(inAccpetProjects.size() + inAccpetProjectsGov.size());
        cockpitStats.setProjectsTotalPerformance(performances.size());
        cockpitStats.setProjectsTotalLogOff(0);
        cockpitStats.setProjectsTotalOperation(inAccpetProjects.size() + inAccpetProjectsGov.size() + operationBaseInfos.size());

        //6.项目效益
        cockpitStats.setExcellentBestAppNum(0);
        cockpitStats.setExcellentMajorAppNum(0);
        cockpitStats.setExcellentCrossLevelSharingNum(0);
        cockpitStats.setExcellentMajorAppNum(0);

        //7 核减资金
        Double reduceFundsTotal = projects.stream().mapToDouble(p -> {
            if(Objects.nonNull(p.getApprovalAmount()) && Objects.nonNull(p.getDeclareAmount())){
                if(Objects.nonNull(p.getContractAmount())){
                    return (p.getDeclareAmount().subtract(p.getContractAmount())).doubleValue();
                }else{
                    return (p.getDeclareAmount().subtract(p.getApprovalAmount())).doubleValue();
                }
            }
            return 0.0;
        }).sum();
        cockpitStats.setReduceFundsTotal(BigDecimal.valueOf(reduceFundsTotal));
        //7.2驳回节约资金
        List<Project> rejectPreProjects = projects.stream().filter(p -> {
            if (Objects.nonNull(p.getStatus()) && p.getStatus().equals(ProjectStatusEnum.PREQUALIFICATION_FAILED.getCode())) {
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList());
        Double rejectPreSum = rejectPreProjects.stream().mapToDouble(p -> {
            if (Objects.nonNull(p.getDeclareAmount())) {
                return p.getDeclareAmount().doubleValue();
            }
            return 0.0;
        }).sum();
        cockpitStats.setRejectSavingsFunds(BigDecimal.valueOf(rejectPreSum));
        //7.3 柱状图
        //7.3.1 申报状态的3个资金
//        projects.stream()
        cockpitStats.setApprovalAmountApproval(BigDecimal.ZERO);
        cockpitStats.setApprovalAmountApprove(BigDecimal.ZERO);
        cockpitStats.setApprovalAmountContract(BigDecimal.ZERO);
        cockpitStats.setDeclaredAmountApproval(BigDecimal.ZERO);
        cockpitStats.setDeclaredAmountApprove(BigDecimal.ZERO);
        cockpitStats.setDeclaredAmountContract(BigDecimal.ZERO);
        cockpitStats.setDifferenceApproval(BigDecimal.ZERO);
        cockpitStats.setDifferenceApprove(BigDecimal.ZERO);
        cockpitStats.setDifferenceContract(BigDecimal.ZERO);

        //8 产出数据
        cockpitStats.setProduceAppNum(0);
        cockpitStats.setProduceDataNum(0);
        cockpitStats.setProduceComponentNum(0);
        cockpitStats.setProduceBrainElementsNum(0);

        //9.绩效
        List<String> projectCodes = projects.stream().map(Project::getProjectCode).collect(Collectors.toList());
        Map<String, String> projectMap = projects.stream().filter(p -> Objects.nonNull(p.getProjectCode())).collect(Collectors.toMap(Project::getProjectCode, Project::getProjectName));
        if(CollUtil.isNotEmpty(projectCodes)){
            List<PerformanceAppraisalProject> performanceAll = performanceAppraisalProjectService.list(Wrappers.lambdaQuery(PerformanceAppraisalProject.class)
                    .in(PerformanceAppraisalProject::getProjectCode, projectCodes));
            List<PerformanceAppraisalProject> verified = performanceAll.stream().filter(p -> {
                        if (Objects.nonNull(p.getVerifyTotalScore())) {
                            return Boolean.TRUE;
                        }
                        return Boolean.FALSE;
                    }).sorted(Comparator.comparing(PerformanceAppraisalProject::getVerifyTotalScore).reversed())
                    .collect(Collectors.toList());

            List<PerformanceAppraisalProject> start5 = performanceAll.stream().filter(p -> {
                        if (Objects.nonNull(p.getVerifyTotalScore())
                                && p.getVerifyTotalScore().compareTo(BigDecimal.valueOf(80)) > 0) {
                            return Boolean.TRUE;
                        }
                        return Boolean.FALSE;
                    }).sorted(Comparator.comparing(PerformanceAppraisalProject::getVerifyTotalScore).reversed())
                    .collect(Collectors.toList());
            List<PerformanceAppraisalProject> start4 = performanceAll.stream().filter(p -> {
                        if (Objects.nonNull(p.getVerifyTotalScore())
                                && p.getVerifyTotalScore().compareTo(BigDecimal.valueOf(60)) >= 0
                                && p.getVerifyTotalScore().compareTo(BigDecimal.valueOf(80)) < 0) {
                            return Boolean.TRUE;
                        }
                        return Boolean.FALSE;
                    }).sorted(Comparator.comparing(PerformanceAppraisalProject::getVerifyTotalScore).reversed())
                    .collect(Collectors.toList());
            List<PerformanceAppraisalProject> start3 = performanceAll.stream().filter(p -> {
                        if (Objects.nonNull(p.getVerifyTotalScore())
                                && p.getVerifyTotalScore().compareTo(BigDecimal.valueOf(40)) >= 0
                                && p.getVerifyTotalScore().compareTo(BigDecimal.valueOf(60)) < 0) {
                            return Boolean.TRUE;
                        }
                        return Boolean.FALSE;
                    }).sorted(Comparator.comparing(PerformanceAppraisalProject::getVerifyTotalScore).reversed())
                    .collect(Collectors.toList());
            List<PerformanceAppraisalProject> start2 = performanceAll.stream().filter(p -> {
                        if (Objects.nonNull(p.getVerifyTotalScore())
                                && p.getVerifyTotalScore().compareTo(BigDecimal.valueOf(20)) >= 0
                                && p.getVerifyTotalScore().compareTo(BigDecimal.valueOf(40)) < 0) {
                            return Boolean.TRUE;
                        }
                        return Boolean.FALSE;
                    }).sorted(Comparator.comparing(PerformanceAppraisalProject::getVerifyTotalScore).reversed())
                    .collect(Collectors.toList());
            List<PerformanceAppraisalProject> start1 = performanceAll.stream().filter(p -> {
                        if (Objects.nonNull(p.getVerifyTotalScore())
                                && p.getVerifyTotalScore().compareTo(BigDecimal.ZERO) >= 0
                                && p.getVerifyTotalScore().compareTo(BigDecimal.valueOf(20)) < 0) {
                            return Boolean.TRUE;
                        }
                        return Boolean.FALSE;
                    }).sorted(Comparator.comparing(PerformanceAppraisalProject::getVerifyTotalScore).reversed())
                    .collect(Collectors.toList());
            cockpitStats.setPerformanceTotal(performanceAll.size());
            cockpitStats.setPerformanceReviewedTotal(verified.size());
            cockpitStats.setPerformanceTobeReviewTotal(performanceAll.size() - verified.size());

            cockpitStats.setPerformanceTop1Score(BigDecimal.valueOf(start1.size()));
            cockpitStats.setPerformanceTop2Score(BigDecimal.valueOf(start2.size()));
            cockpitStats.setPerformanceTop3Score(BigDecimal.valueOf(start3.size()));
            cockpitStats.setPerformanceTop4Score(BigDecimal.valueOf(start4.size()));
            cockpitStats.setPerformanceTop5Score(BigDecimal.valueOf(start5.size()));
        }

        return cockpitStats;
    }

    /**
     * 计算环节滞后项目
     * @param projects
     * @return
     */
    private Integer computeLagging(List<Project> projects) {
        List<String> projectCodes = projects.stream().map(Project::getProjectCode).collect(Collectors.toList());
        Map<String, Project> map = projects.stream().collect(Collectors.toMap(p -> p.getProjectCode(), p -> p));
        List<WflowEarlyWarningRecords> records = Lists.newArrayList();
        if(CollUtil.isNotEmpty(projectCodes)){
            records = earlyWarningRecordsService.list(Wrappers.lambdaQuery(WflowEarlyWarningRecords.class)
                    .in(WflowEarlyWarningRecords::getProjectCode, projectCodes));
        }
        return records.stream().mapToInt(r -> {
            if(!map.containsKey(r.getProjectCode())){
                return 0;
            }
            Project project = map.get(r.getProjectCode());
            //如果状态 相等 说明预警了 但是没处理 +1
            if(Objects.nonNull(project.getStatus()) && Objects.nonNull(r.getProjectStatus()) &&
                    project.getStatus().equals(r.getProjectStatus())){
                return 1;
            }
            return 0;
        }).sum();
    }

    private void buidMonitorRise(CockpitStats cockpitStats, CockpitStats thisYearData, CockpitStats lastYearData) {
        if(Objects.isNull(thisYearData) || Objects.isNull(lastYearData)){
            return;
        }
        cockpitStats.setMonitorOverdueConstructionProjectsRise(BizUtils.computeRise(thisYearData.getMonitorOverdueConstructionProjectsNum(),
                lastYearData.getMonitorOverdueConstructionProjectsNum()));
        cockpitStats.setMonitorLaggingProjectsRise(BizUtils.computeRise(thisYearData.getMonitorLaggingProjectsNum(),
                lastYearData.getMonitorLaggingProjectsNum()));
        cockpitStats.setMonitorRejectedApproveProjectsRise(BizUtils.computeRise(thisYearData.getMonitorRejectedApproveProjectsNum(),
                lastYearData.getMonitorRejectedApproveProjectsNum()));
        cockpitStats.setMonitorRejectedPreReviewProjectsRise(BizUtils.computeRise(thisYearData.getMonitorRejectedPreReviewProjectsNum(),
                lastYearData.getMonitorRejectedPreReviewProjectsNum()));
        cockpitStats.setMonitorAcceptConditionsNotStandardsRise(BizUtils.computeRise(thisYearData.getMonitorAcceptConditionsNotStandardsNum(),
                lastYearData.getMonitorAcceptConditionsNotStandardsNum()));
    }
}
